@@ -1,10 +1,10 @@
 from statefun_tasks.serialisation import DefaultSerialiser
-from statefun_tasks.messages_pb2 import TaskState, TaskRequest, PipelineState
+from statefun_tasks.messages_pb2 import TaskState, TaskRequest, PipelineState, Address
 from statefun_tasks.protobuf import pack_any
 
 from statefun import kafka_egress_record
 from statefun.request_reply import BatchContext
-from statefun.request_reply_pb2 import Address
+from datetime import timedelta
 
 
 class TaskContext(object):
@@ -78,6 +78,14 @@ class TaskContext(object):
         """
         return self._task_meta.get('pipeline_id', None)
 
+    def get_pipeline_address(self):
+        """
+        Address of the pipeline if this task is called as part of a pipeline else None
+
+        :return: pipeline ID
+        """
+        return self._task_meta.get('pipeline_address', None)
+
     def get_parent_task_id(self):
         """
         ID of the parent task if this task has one else None
@@ -126,7 +134,10 @@ class TaskContext(object):
     def pack_and_save(self, key:str, value):
         self._context.state(key).pack(value)
 
-    def pack_and_send(self, destination, task_id, request):
+    def pack_and_send(self, destination, task_id, request, delay :timedelta=None):
+        if delay:
+            return self.pack_and_send_after(delay, destination, task_id, request)
+            
         self._context.pack_and_send(destination, task_id, request)
 
     def pack_and_send_after(self, delay, destination, task_id, request):
