@@ -10,6 +10,7 @@ class EventHandlers(object):
         self._on_pipeline_created_handlers = []
         self._on_pipeline_status_changed = []
         self._on_pipeline_task_finished_handlers = []
+        self._on_pipeline_finished_handlers = []
 
     def on_task_started(self, handler):
         self._on_task_started_handlers.append(handler)
@@ -29,6 +30,9 @@ class EventHandlers(object):
     def on_pipeline_task_finished(self, handler):
         self._on_pipeline_task_finished_handlers.append(handler)
 
+    def on_pipeline_finished(self, handler):
+        self._on_pipeline_finished_handlers.append(handler)
+
     def notify_task_started(self, context, task_request: TaskRequest):
         self._notify_all(self._on_task_started_handlers, context, task_request)
 
@@ -52,6 +56,15 @@ class EventHandlers(object):
 
         self._notify_all(self._on_pipeline_task_finished_handlers, context, task_result=task_result, task_exception=task_exception)
 
+    def notify_pipeline_finished(self, context, pipeline, task_result_or_exception):
+        if isinstance(task_result_or_exception, TaskResult):
+            task_result, task_exception = task_result_or_exception, None
+        else:
+            task_result, task_exception = None, task_result_or_exception
+
+        self._notify_all(self._on_pipeline_finished_handlers, context, pipeline, task_result=task_result, task_exception=task_exception)
+
+
     @staticmethod
     def _notify_all(handlers, *args, **kwargs):
         for handler in handlers:
