@@ -67,10 +67,11 @@ def enable_inline_tasks(tasks: FlinkTasks):
     _log.warning('Inline tasks enabled. This is a potential security risk')
 
 
-def inline_task(include=None, with_context=False, with_state=False, **params):
+def inline_task(include=None, depends=None, with_context=False, with_state=False, **params):
     """
     Declares an inline Flink task
     :param include: list of modules to include with the pickled code
+    :param depends: list of tasks this tasks depends on (i.e. will invoke as a pipeline).  Includes from these tasks will be added automatically
     :param with_context: If set the first parameter to the function is exepcted to be the task context
     :param with_state: If set the next parameter is expected to be the task state
     :param params: any additional parameters to the Flink Task (such as a retry policy)
@@ -78,20 +79,41 @@ def inline_task(include=None, with_context=False, with_state=False, **params):
     """
     
     includes = include or []
+    depends = depends or []
+
+    def _includes():
+        all_includes = set(includes)
+
+        for dependency in depends:
+            all_includes.update(dependency.includes())
+
+        return list(all_includes)
 
     def pickle(fn):
-        for module in includes:
-            cloudpickle.register_pickle_by_value(module)
+        modules = _includes()
+
+        for module in modules:
+            try:
+                cloudpickle.register_pickle_by_value(module)
+            except ValueError:
+                # ignore cloudpickle errors about modules not yet being imported
+                pass
 
         code = cloudpickle.dumps(fn)
 
-        for module in includes:
-            cloudpickle.unregister_pickle_by_value(module)
+        for module in modules:
+            try:
+                cloudpickle.unregister_pickle_by_value(module)
+            except ValueError:
+                # ignore cloudpickle errors about modules not yet being imported
+                pass
 
         return code
 
     def decorator(fn):
 
+        params.setdefault('display_name', f'{fn.__module__}.{fn.__name__}')
+
         def send(*args, **kwargs):
 
             if __defaults is None:
@@ -103,8 +125,8 @@ def inline_task(include=None, with_context=False, with_state=False, **params):
                 pass
 
             fn_kwargs = {**kwargs, '__with_context': with_context, '__with_state': with_state, '__code': code}
-            display_name = params.setdefault(f'{fn.__module__}.{fn.__name__}')
-            return FlinkTasks.extend(run_code, **__defaults).send(*args, **fn_kwargs).set(display_name=display_name)
+            task_params = {**__defaults, **params}
+            return FlinkTasks.extend(run_code, **task_params).send(*args, **fn_kwargs)
 
         def to_task(args, kwargs, is_finally=False, parameters=None):
 
@@ -117,9 +139,10 @@ def inline_task(include=None, with_context=False, with_state=False, **params):
                 pass
 
             fn_kwargs = {**kwargs, '__with_context': with_context, '__with_state': with_state, '__code': code}
-            params.setdefault(f'{fn.__module__}.{fn.__name__}')
-            return FlinkTasks.extend(run_code, **__defaults).to_task(args, fn_kwargs, is_finally, params)
+            task_params = {**__defaults, **params}
+            return FlinkTasks.extend(run_code, **task_params).to_task(args, fn_kwargs, is_finally, {})
 
+        fn.includes = _includes
         fn.send = send
         fn.to_task = to_task
 
