@@ -3,6 +3,7 @@ from statefun_tasks.pipeline_impl.handlers import PipelineMessageHandler
 from statefun_tasks.messages_pb2 import TaskRequest, TaskResult, TaskException, TaskStatus, PipelineState, ChildPipeline, TaskInfo
 from statefun_tasks.serialisation import pack_any
 from statefun_tasks.utils import _gen_id
+from statefun_tasks.type_helpers import _create_task_result
 from google.protobuf.any_pb2 import Any
 from typing import Union
 
@@ -13,24 +14,24 @@ class BeginPipelineHandler(PipelineMessageHandler):
     
     def can_handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException]) -> bool:
         return (context.pipeline_state is None or context.pipeline_state.status.value == TaskStatus.PENDING) \
-            and isinstance(message, TaskRequest) \
-                and not self.graph.is_empty()
+            and isinstance(message, TaskRequest)
 
-    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline: '_Pipeline', task_state: Any):
+    async def handle_message(self, context: TaskContext, message: Union[TaskRequest, TaskResult, TaskException], pipeline: '_Pipeline', state: Any):
         invoking_task = message  # type: TaskRequest
 
         # ensure we pick up the correct caller id when task producing this pipeline is a retry
-        if context.task_state.original_caller_id == '':
+        task_state = context.task_state.by_uid[invoking_task.uid]
+        if task_state.original_caller_id == '':
             caller_id = context.get_caller_id()
         else:
-            caller_id = context.task_state.original_caller_id
+            caller_id = task_state.original_caller_id
 
         # 1. record all the continuations into a pipeline and save into state with caller id and address
         context.pipeline_state = PipelineState(id = context.get_task_id(), address = context.get_address())
         context.pipeline_state.status.value = TaskStatus.RUNNING
         context.pipeline_state.pipeline.CopyFrom(pipeline.to_proto())
         context.pipeline_state.is_fruitful = pipeline.is_fruitful
-        context.pipeline_state.task_state.CopyFrom(pack_any(self.serialiser.to_proto(task_state)))
+        context.pipeline_state.task_state.CopyFrom(pack_any(self.serialiser.to_proto(state)))
         context.pipeline_state.invocation_id = _gen_id()
 
         if caller_id is not None:
@@ -46,19 +47,33 @@ class BeginPipelineHandler(PipelineMessageHandler):
         # and notify root pipeline of a new child pipeline
         self._notify_pipeline_created(context)
 
-        # 2. get initial tasks(s) to call - might be single start of chain task or a group of tasks to call in parallel
-        tasks, max_parallelism, slice = self.graph.get_initial_tasks()
+        # if we got an empty pipeline then complete immediately
+        if self._graph.is_empty():
+            task_result = _create_task_result(message)
+            task_result.invocation_id = context.pipeline_state.invocation_id
+            self._serialiser.serialise_result(task_result, ([]), state)
 
-        # 2a. if we skipped over empty group(s) then make sure we pass empty array to next task (result of in_parallel([]) is intuitively [])
-        if slice > 0:
-            for task in tasks:
-                task.request = self._serialiser.serialise_args_and_kwargs(([]), {})
+            context.pipeline_state.status.value = TaskStatus.COMPLETED
+            pipeline.events.notify_pipeline_status_changed(context, context.pipeline_state.pipeline, context.pipeline_state.status.value)
+            
+            # continue
+            return True, task_result
 
-        # 3. split into tasks to call now and those to defer if max parallelism is exceeded
-        await self.submitter.submit_tasks(context, tasks, max_parallelism=max_parallelism)
+        else:
+            # else get initial tasks(s) to call - might be single start of chain task or a group of tasks to call in parallel
+            tasks, max_parallelism, slice = self.graph.get_initial_tasks()
 
-        # 4. break
-        return False, message
+            # if we skipped over empty group(s) then make sure we pass empty array to next task (result of in_parallel([]) is intuitively [])
+            if slice > 0:
+                for task in tasks:
+                    _, kwargs = self._serialiser.deserialise_args_and_kwargs(task.request)
+                    task.request = self._serialiser.serialise_args_and_kwargs(([]), kwargs)
+
+            # split into tasks to call now and those to defer if max parallelism is exceeded
+            await self.submitter.submit_tasks(context, tasks, max_parallelism=max_parallelism)
+
+            # break
+            return False, message
 
     def _notify_pipeline_created(self, context):
         # if this pipeline is the already root do nothing as it's not a child
