@@ -20,7 +20,8 @@ class ContinuePipelineHandler(PipelineMessageHandler):
         caller_uid = message.uid
 
         # mark pipeline step as complete
-        self.graph.mark_task_complete(caller_uid, task_result_or_exception)
+        if not self.graph.mark_task_complete(caller_uid, task_result_or_exception):
+            return False, task_result_or_exception
 
         # notify event handler (with option to cancel)
         try:
@@ -37,7 +38,7 @@ class ContinuePipelineHandler(PipelineMessageHandler):
 
         # if this task is part group then we need to record the results so we can aggregate later
         if group is not None:
-            await self.result_aggregator.add_result(context, group, caller_uid, task_result_or_exception)
+            await self.result_aggregator.add_result(context, caller_uid, task_result_or_exception)
 
             # once the group is complete aggregate the results
             if group.is_complete():
