@@ -86,7 +86,6 @@ class DeferredTaskSubmitter(object):
                 context.pack_and_send(paused_task.destination, paused_task.task_request.id, paused_task.task_request)
 
         context.pipeline_state.ClearField('paused_tasks')
-        await self._cleanup_storage(context)
 
     def _create_task_deferral(self, context, tasks_to_defer, task_result_or_exception):
         # create task deferral to hold these deferred tasks
@@ -195,12 +194,3 @@ class DeferredTaskSubmitter(object):
         paused_task = PausedTask()
         paused_task.ParseFromString(proto_bytes)
         return paused_task
-
-    async def _cleanup_storage(self, context):
-        if self._storage is None:
-            return
-
-        try:
-            await self._storage.delete([context.pipeline_state.invocation_id])
-        except Exception as ex:
-            _log.warning(f'Error cleaning up paused tasks for {context.pipeline_state.invocation_id} from backend storage - {ex}')
