@@ -4,8 +4,8 @@ from statefun_tasks.pipeline_builder import PipelineBuilder
 
 from statefun_tasks.types import Task, RetryPolicy
 
-from statefun_tasks.messages_pb2 import ChildPipeline, TaskRequest, TaskResult, TaskException, TaskActionRequest
-from statefun_tasks.type_helpers import _create_task_exception
+from statefun_tasks.messages_pb2 import ChildPipeline, TaskRequest, TaskResult, TaskException, TaskActionRequest, Address
+from statefun_tasks.type_helpers import _create_task_result, _create_task_exception
 from statefun_tasks.context import TaskContext
 from statefun_tasks.utils import _task_type_for, _unpack_single_tuple_args, _gen_id
 from statefun_tasks.pipeline import _Pipeline
@@ -15,6 +15,7 @@ from statefun_tasks.events import EventHandlers
 
 from statefun.request_reply import BatchContext
 from typing import Union
+from datetime import timedelta
 import logging
 
 _log = logging.getLogger('FlinkTasks')
@@ -241,6 +242,65 @@ class FlinkTasks(object):
                 'Expected function to have a send attribute. Make sure it is decorated with @tasks.bind()')
         return send_func(*args, **kwargs)
 
+    def clone_task_request(self, context: TaskContext) -> TaskRequest:
+        """
+        Returns a complete copy of the TaskRequest associated with this TaskContext with the 
+        caller address details extracted as necessary from the context
+
+        :param context: TaskContext
+        :return: a TaskRequest
+        """
+        task_request = context.unpack('task_request', TaskRequest)
+
+        # copy caller details from context unless explictly set already
+        if not task_request.HasField('reply_address') and not task_request.HasField('reply_topic'):
+
+            address, caller_id = self._get_caller_address_and_id(context, task_request)
+            
+            if address is not None and caller_id is not None:
+                namespace, function_type = address.split('/')
+                task_request.reply_address.CopyFrom(Address(namespace=namespace, type=function_type, id=caller_id))
+
+        return task_request
+
+    def unpack_task_request(self, task_request: TaskRequest) -> tuple:
+        """
+        Unpacks a TaskRequest into args, kwargs and state
+
+        :param task_request: TaskRequest
+        :return: args, kwargs and state from this task_request
+        """
+        args, kwargs, state = self._serialiser.deserialise_request(task_request)
+        return args, kwargs, state
+
+    def send_result(self, context: TaskContext, task_request: TaskRequest, result, state=Ellipsis, delay: timedelta=None):
+        """
+        Sends a result
+        :param context: TaskContext
+        :param task_request: the incoming TaskRequest
+        :param result: the result(s) to return
+        :param state: the state to include in the result.  If not specified it will be copied from the TaskRequest
+        """
+        task_result = _create_task_result(task_request)
+
+        if state == Ellipsis:  # default value since None is perfectly valid state
+            state = task_request.state
+
+        self._serialiser.serialise_result(task_result, result, state)
+
+        self.emit_result(context, task_request, task_result, delay)
+
+    def fail(self, context, task_input, ex, delay: timedelta=None):
+        """
+        Sends a failure
+        :param context: TaskContext
+        :param task_input: the incoming TaskRequest or TaskActionRequest
+        :param ex: the exception to return
+        """
+        task_exception = _create_task_exception(task_input, ex)
+
+        self.emit_result(context, task_input, task_exception, delay)
+
     def get_pipeline(self, context):
         pipeline_protos = context.pipeline_state.pipeline
 
@@ -255,8 +315,8 @@ class FlinkTasks(object):
         except:
             return None
 
-    def emit_result(self, context, task_input, task_result):
-        # set invocation id
+    def emit_result(self, context, task_input, task_result, delay: timedelta=None):
+        # copy over invocation id
         if isinstance(task_result, (TaskResult, TaskException)):
             task_result.invocation_id = task_input.invocation_id
 
@@ -267,22 +327,22 @@ class FlinkTasks(object):
         # or call back to a particular flink function if reply_address was specified
         elif task_input.HasField('reply_address'):
             address, identifer = context.to_address_and_id(task_input.reply_address)
-            context.pack_and_send(address, identifer, task_result)
+            context.pack_and_send(address, identifer, task_result, delay)
 
         elif isinstance(task_input, TaskRequest):
-            # if this was a task request it could be part of a pipeline - i.e. called from another tasks so
-            # either call back to original caller (e.g. if this is a result of a retry)
-            if context.task_state.original_caller_id != '':
-                context.pack_and_send(context.task_state.original_caller_address, context.task_state.original_caller_id, task_result)
+            address, caller_id = self._get_caller_address_and_id(context, task_input)
+            if address is not None and caller_id is not None:
+                context.pack_and_send(address, caller_id, task_result, delay)
 
-            # or call back to our caller (if there is one)
-            elif context.get_caller_id() is not None:
-                context.pack_and_reply(task_result)
+        # clean up
+        if task_input.uid in context.task_state.by_uid:
+            del context.task_state.by_uid[task_input.uid]
 
-    def fail(self, context, task_input, ex):
-        task_exception = _create_task_exception(task_input, ex)
+    def _get_caller_address_and_id(self, context, task_request):
+        task_state = context.task_state.by_uid[task_request.uid]
+        
+        # either as original caller (e.g. if this is a result of a retry) or context.get_caller_...() otherwise
+        address = task_state.original_caller_address if task_state.original_caller_address != '' else context.get_caller_address()
+        caller_id = task_state.original_caller_id if task_state.original_caller_id != '' else context.get_caller_id()
 
-        context.delete('task_result')
-        context.pack_and_save('task_exception', task_exception)
-
-        self.emit_result(context, task_input, task_exception)
+        return address, caller_id
